@@ -32,6 +32,14 @@
 
 #define MIME_ERROR g_quark_from_static_string("mime-error-quark")
 
+#ifndef PATH_SEPARATOR
+# ifdef _WIN32
+#  define PATH_SEPARATOR ";"
+# else
+#  define PATH_SEPARATOR ":"
+# endif
+#endif
+
 /* This is the list of directories to scan when finding old type files to
  * delete. It is also used to warn about invalid MIME types.
  */
@@ -45,6 +53,7 @@ const char *media_types[] = {
 	"message",
 	"model",
 	"multipart",
+	"x-epoc",
 };
 
 /* Represents a MIME type */
@@ -512,16 +521,16 @@ static void load_source_file(const char *filename)
 
 	if (root->ns == NULL || xmlStrcmp(root->ns->href, FREE_NS) != 0)
 	{
-		g_error("* Wrong namespace on document element\n"
-			"*   in '%s'\n"
-			"*   (should be %s)\n", filename, FREE_NS);
+		g_warning("* Wrong namespace on document element\n"
+			  "*   in '%s'\n"
+			  "*   (should be %s)\n", filename, FREE_NS);
 		goto out;
 	}
 	
 	if (strcmp((char *)root->name, "mime-info") != 0)
 	{
-		g_error("* Root element <%s> is not <mime-info>\n"
-			"*   (in '%s')\n", root->name, filename);
+		g_warning("* Root element <%s> is not <mime-info>\n"
+			  "*   (in '%s')\n", root->name, filename);
 		goto out;
 	}
 
@@ -565,12 +574,12 @@ static void load_source_file(const char *filename)
 
 		if (error)
 		{
-			g_error("* Error in type '%s/%s'\n"
-				"*   (in %s):\n"
-				"*   %s.\n",
-				type ? type->media : _("unknown"),
-				type ? type->subtype : _("unknown"),
-				filename, error->message);
+			g_warning("* Error in type '%s/%s'\n"
+				  "*   (in %s):\n"
+				  "*   %s.\n",
+				  type ? type->media : _("unknown"),
+				  type ? type->subtype : _("unknown"),
+				  filename, error->message);
 			g_error_free(error);
 		}
 	}
@@ -674,9 +683,9 @@ static void write_out_glob(gpointer key, gpointer value, gpointer data)
 	for ( ; list; list = list->next) {
 		type = (Type *)list->data;
 		if (strchr(pattern, '\n'))
-			g_error("* Glob patterns can't contain literal newlines "
-				"(%s in type %s/%s)\n", pattern,
-				type->media, type->subtype);
+			g_warning("* Glob patterns can't contain literal newlines "
+				  "(%s in type %s/%s)\n", pattern,
+				  type->media, type->subtype);
 		else
 			g_fprintf(stream, "%s/%s:%s\n",
 				type->media, type->subtype, pattern);
@@ -695,8 +704,12 @@ static void atomic_update(const gchar *pathname)
 
 	new_name = g_strndup(pathname, len - 4);
 
+#ifdef _WIN32
+	/* we need to remove the old file first! */
+	remove(new_name);
+#endif
 	if (rename(pathname, new_name))
-		g_warning("Failed to rename %s as %s\n", pathname, new_name);
+		g_warning("Failed to rename %s as %s , errno: %d\n", pathname, new_name, errno);
 
 	g_free(new_name);
 }
@@ -709,7 +722,11 @@ static void write_out_type(gpointer key, gpointer value, gpointer data)
 	char *media, *filename;
 
 	media = g_strconcat(mime_dir, "/", type->media, NULL);
+#ifdef _WIN32
+	mkdir(media);
+#else
 	mkdir(media, 0755);
+#endif
 
 	filename = g_strconcat(media, "/", type->subtype, ".xml.new", NULL);
 	g_free(media);
@@ -1486,15 +1503,15 @@ static void check_in_path_xdg_data(const char *mime_path)
 
 	if (stat(path, &path_info))
 	{
-		g_error("Can't stat '%s' directory: %s",
+		g_warning("Can't stat '%s' directory: %s",
 				path, g_strerror(errno));
 		goto out;
 	}
 
 	env = getenv("XDG_DATA_DIRS");
 	if (!env)
-		env = "/usr/local/share/:/usr/share/";
-	dirs = g_strsplit(env, ":", 0);
+		env = "/usr/local/share/"PATH_SEPARATOR"/usr/share/";
+	dirs = g_strsplit(env, PATH_SEPARATOR, 0);
 	g_return_if_fail(dirs != NULL);
 	for (n = 0; dirs[n]; n++)
 		;
@@ -1649,7 +1666,7 @@ write_map_entry (gpointer key,
       offset = GPOINTER_TO_UINT (g_hash_table_lookup (map_data->pool, values[i]));
       if (offset == 0)
 	{
-	  g_error ("Missing string: '%s'\n", values[i]);
+	  g_warning ("Missing string: '%s'\n", values[i]);
 	  map_data->error = TRUE;  
 	}
       
@@ -1777,7 +1794,7 @@ write_parent_entry (gpointer key,
   offset = GPOINTER_TO_UINT (g_hash_table_lookup (map_data->pool, mimetype));
   if (offset == 0)
     {
-      g_error ("Missing string: '%s'\n", (gchar *)key);
+      g_warning ("Missing string: '%s'\n", (gchar *)key);
       map_data->error = TRUE;  
     }
 
@@ -1810,7 +1827,7 @@ write_parent_list (gpointer key,
       offset = GPOINTER_TO_UINT (g_hash_table_lookup (map_data->pool, parent));
       if (offset == 0)
 	{
-	  g_error ("Missing string: '%s'\n", parent);
+	  g_warning ("Missing string: '%s'\n", parent);
 	  map_data->error = TRUE;  
 	}
       
@@ -2031,7 +2048,7 @@ build_suffixes (gpointer key,
       
       if (suffix == NULL)
 	{
-	  g_error ("Glob '%s' is not valid UTF-8\n", glob);
+	  g_warning ("Glob '%s' is not valid UTF-8\n", glob);
 	  return;
 	}
 
@@ -2095,7 +2112,7 @@ write_suffix_entries (FILE        *cache,
       offset = GPOINTER_TO_UINT(g_hash_table_lookup (strings, entry->mimetype));
       if (offset == 0)
 	{
-	  g_error ("Missing string: '%s'\n", entry->mimetype);
+	  g_warning ("Missing string: '%s'\n", entry->mimetype);
 	  return FALSE;
 	}
     }
@@ -2189,7 +2206,7 @@ write_match (gpointer key,
   offset = GPOINTER_TO_UINT (g_hash_table_lookup (mdata->strings, mimetype));
   if (offset == 0)
     {
-      g_error ("Missing string: '%s'\n", mimetype);
+      g_warning ("Missing string: '%s'\n", mimetype);
       g_free (mimetype);
       mdata->error = TRUE;
       return;
@@ -2573,7 +2590,7 @@ write_cache (FILE *cache)
   offset = 0;
   if (!write_header (cache, 0, 0, 0, 0, 0, 0, 0, &offset))
     {
-      g_error ("Failed to write header\n");
+      g_warning ("Failed to write header\n");
       return FALSE;
     }
 
@@ -2583,7 +2600,7 @@ write_cache (FILE *cache)
 
   if (!write_strings (cache, strings, &offset))
     {
-      g_error ("Failed to write strings\n");
+      g_warning ("Failed to write strings\n");
       return FALSE;
     }
   g_message ("Wrote %d strings at %x - %x\n", 
@@ -2592,7 +2609,7 @@ write_cache (FILE *cache)
   alias_offset = offset;
   if (!write_alias_cache (cache, strings, &offset))
     {
-      g_error ("Failed to write alias list\n");
+      g_warning ("Failed to write alias list\n");
       return FALSE;
     }
   g_message ("Wrote aliases at %x - %x\n", alias_offset, offset);
@@ -2600,7 +2617,7 @@ write_cache (FILE *cache)
   parent_offset = offset;
   if (!write_parent_cache (cache, strings, &offset))
     {
-      g_error ("Failed to write parent list\n");
+      g_warning ("Failed to write parent list\n");
       return FALSE;
     }
   g_message ("Wrote parents at %x - %x\n", parent_offset, offset);
@@ -2608,7 +2625,7 @@ write_cache (FILE *cache)
   literal_offset = offset;
   if (!write_literal_cache (cache, strings, &offset))
     {
-      g_error ("Failed to write literal list\n");
+      g_warning ("Failed to write literal list\n");
       return FALSE;
     }
   g_message ("Wrote literal globs at %x - %x\n", literal_offset, offset);
@@ -2616,7 +2633,7 @@ write_cache (FILE *cache)
   suffix_offset = offset;
   if (!write_suffix_cache (cache, strings, &offset))
     {
-      g_error ("Failed to write suffix list\n");
+      g_warning ("Failed to write suffix list\n");
       return FALSE;
     }
   g_message ("Wrote suffix globs at %x - %x\n", suffix_offset, offset);
@@ -2624,7 +2641,7 @@ write_cache (FILE *cache)
   glob_offset = offset;
   if (!write_glob_cache (cache, strings, &offset))
     {
-      g_error ("Failed to write glob list\n");
+      g_warning ("Failed to write glob list\n");
       return FALSE;
     }
   g_message ("Wrote full globs at %x - %x\n", glob_offset, offset);
@@ -2632,7 +2649,7 @@ write_cache (FILE *cache)
   magic_offset = offset;
   if (!write_magic_cache (cache, strings, &offset))
     {
-      g_error ("Failed to write magic list\n");
+      g_warning ("Failed to write magic list\n");
       return FALSE;
     }
   g_message ("Wrote magic at %x - %x\n", magic_offset, offset);
@@ -2640,7 +2657,7 @@ write_cache (FILE *cache)
   namespace_offset = offset;
   if (!write_namespace_cache (cache, strings, &offset))
     {
-      g_error ("Failed to write namespace list\n");
+      g_warning ("Failed to write namespace list\n");
       return FALSE;
     }
   g_debug ("Wrote namespace list at %x - %x\n", namespace_offset, offset);
@@ -2653,7 +2670,7 @@ write_cache (FILE *cache)
 		     suffix_offset, glob_offset, magic_offset, 
 		     namespace_offset, &offset))
     {
-      g_error ("Failed to rewrite header\n");
+      g_warning ("Failed to rewrite header\n");
       return FALSE;
     }
 
@@ -2734,7 +2751,7 @@ int main(int argc, char **argv)
 
 	if (access(mime_dir, W_OK))
 	{
-		g_error(_("%s: I don't have write permission on %s.\n"
+		g_warning(_("%s: I don't have write permission on %s.\n"
 			     "Try rerunning me as root.\n"), argv[0], mime_dir);
 		return EXIT_FAILURE;
 	}
@@ -2811,6 +2828,7 @@ int main(int argc, char **argv)
 		ns_path = g_strconcat(mime_dir, "/XMLnamespaces.new", NULL);
 		stream = open_or_die(ns_path);
 		write_namespaces(stream);
+		fclose(stream);
 
 		atomic_update(ns_path);
 		g_free(ns_path);
@@ -2823,6 +2841,7 @@ int main(int argc, char **argv)
 		path = g_strconcat(mime_dir, "/subclasses.new", NULL);
 		stream = open_or_die(path);
 		write_subclasses(stream);
+		fclose(stream);
 
 		atomic_update(path);
 		g_free(path);
@@ -2835,6 +2854,7 @@ int main(int argc, char **argv)
 		path = g_strconcat(mime_dir, "/aliases.new", NULL);
 		stream = open_or_die(path);
 		write_aliases(stream);
+		fclose(stream);
 
 		atomic_update(path);
 		g_free(path);
@@ -2847,6 +2867,7 @@ int main(int argc, char **argv)
 		path = g_strconcat(mime_dir, "/mime.cache.new", NULL);
 		stream = open_or_die(path);
 		write_cache(stream);
+		fclose(stream);
 
 		atomic_update(path);
 		g_free(path);
